/*
Phenotyper - a tool for collecting phenotyping data using mobile terminals
Copyright (C) 2015,  jgremmels(at)bioinformatics.org

Phenotyper is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Contributors:
jgremmels(at)bioinformatics.org - initial API and implementation
*/

package mpimp.phenotypingschemecomposer.util;

import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.util.ArrayList;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import mpimp.phenotypingschemecomposer.db.TPCModel;
import mpimp.phenotypingschemecomposer.db.TestProgramEntityRecord;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XMLReader {

	// <?xml version="1.0" encoding="utf-16"?>
	// <TESTPROGRAM ID="" NAME="" VERSION="" OBJECT="" DESCRIPTION=""
	// OBJECT_TYPE="">
	// <PARAMETER ID="" NAME_E="" NAME_D="">
	// <ATTRIBUTE ID="" NAME_E="" NAME_D="" ORDER_NUMBER="">
	// <VALUE NAME_E="" NAME_D="" IMAGE_FILE="" />
	// </ATTRIBUTE>
	// <VALUE_GROUP ID="">
	// <ATTRIBUTE ID="" NAME_E="" NAME_D="" ORDER_NUMBER="">
	// <VALUE NAME_E="" NAME_D="" IMAGE_FILE="" />
	// </VALUE_GROUP>
	// </PARAMETER>
	// </TESTPROGRAM>

	private static final String tagNameTestProgram = "TESTPROGRAM";
	private static final String tagNameParameter = "PARAMETER";
	private static final String tagNameAttribute = "ATTRIBUTE";
	private static final String attributeNameID = "ID";
	private static final String attributeNameName = "NAME";
	private static final String attributeNameVersion = "VERSION";
	private static final String attributeNameObject = "OBJECT";
	private static final String attributeNameDescription = "DESCRIPTION";
	private static final String attributeNameObjectType = "OBJECT_TYPE";
	private static final String attributeNameOrderNumber = "ORDER_NUMBER";

	public static void readTestProgramFile(TPCModel tpcModel)
			throws Exception {

		File xmlFile = null;
		try {
			xmlFile = new File(tpcModel.getSourceFile());
		} catch (Exception e) {
			String message = "Can not open file %s. Reason: %s";
			message = String.format(message, tpcModel.getSourceFile(), e.getMessage());
			throw new Exception(message);
		}

		String webApplicationParentPath = TPCProperties.getInstance()
				.getProperty("webApplicationParentPath");
		
		URL urlSchema = new URL("file:///" + webApplicationParentPath + File.separator + "TestProgramComposer"
		+ File.separator + "WEB-INF" + File.separator + "classes" + File.separator + "Testprogram.xsd");
	    Schema testProgramSchema = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI).newSchema(urlSchema);
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		factory.setValidating(false);
		factory.setSchema(testProgramSchema);
		
		DocumentBuilder builder = null;
		builder = factory.newDocumentBuilder();
		tpcModel.setXmlReadingErrors(""); // ensure that there are no old parsing errors
		builder.setErrorHandler(new TPCXMLErrorHandler(tpcModel));
		Document doc;

		try {
			doc = builder.parse(xmlFile);
			
			if (!tpcModel.getXmlReadingErrors().equals("")) {
				new FileInputStream(xmlFile).close();
				return;
			}
			
			Element programElement = doc.getDocumentElement();

			if (programElement != null && programElement.getTagName().equals(tagNameTestProgram)) {
				String idString = programElement
						.getAttribute(attributeNameID);
				Integer testProgramId = Integer.parseInt(idString);
				String name = programElement
						.getAttribute(attributeNameName);
				String version = programElement
						.getAttribute(attributeNameVersion);
				String object = programElement
						.getAttribute(attributeNameObject);
				String description = programElement
						.getAttribute(attributeNameDescription);
				Attr testObjectTypeNode = XMLUtilities.getAttributeNodeByName(
						programElement, attributeNameObjectType);
				
				Integer testObjectType = 0;
				// Attribute may be not present in older versions
				if (testObjectTypeNode != null) {
					try {
						testObjectType = Integer.parseInt(testObjectTypeNode
								.getValue());
					} catch (NumberFormatException nfe) {
						// do nothing - we are dealing with an old version of result
						// file
//						String message = "Non parsable value ("
//								+ testObjectTypeNode.getValue()
//								+ ") for test object type.";
//						errorMessages.add(message);
					}
				} else {
//					String message = "Warning: you are using a result file produced by an old version of Phenotyper!";
//					errorMessages.add(message);
				}
				
				tpcModel.setCurrentTestProgramName(name);
				tpcModel.setCurrentTestProgramVersion(version);
				tpcModel.setCurrentTestProgramMessobjekt(object);
				tpcModel.setCurrentTestProgramDescription(description);
				tpcModel.setCurrentTestObjectType(testObjectType);

				ArrayList<TestProgramEntityRecord> testProgramEntityRecords = new ArrayList<TestProgramEntityRecord>();

				NodeList parameters = XMLUtilities.getNodeListByTagName(
						programElement, tagNameParameter);

				for (int p = 0; p < parameters.getLength(); p++) {
					Element parameterElement = (Element) parameters.item(p);
					String parameterIdString = parameterElement
							.getAttribute(attributeNameID);
					Integer parameterId = Integer.parseInt(parameterIdString);

					NodeList attributes = XMLUtilities.getNodeListByTagName(
							parameterElement, tagNameAttribute);

					for (int a = 0; a < attributes.getLength(); a++) {
						TestProgramEntityRecord currentTestProgramEntityRecord = new TestProgramEntityRecord();

						Element attributeElement = (Element) attributes.item(a);
						String attributeIdString = attributeElement
								.getAttribute(attributeNameID);
						Integer attributeId = Integer
								.parseInt(attributeIdString);
						String orderNumberString = attributeElement
								.getAttribute(attributeNameOrderNumber);
						Integer orderNumber = 0;
						if (!orderNumberString.equals("")) { //order number is not mandatory
							orderNumber = Integer.parseInt(orderNumberString);
						} 
						currentTestProgramEntityRecord
								.setTestprogramm(testProgramId);
						currentTestProgramEntityRecord.setEntity(parameterId);
						currentTestProgramEntityRecord.setValue(attributeId);
						currentTestProgramEntityRecord.setOrderNumber(orderNumber);

						testProgramEntityRecords
								.add(currentTestProgramEntityRecord);
					}
				}

				tpcModel.setTestProgramEntityRecords(testProgramEntityRecords);

			} else {
				String message = "File %s seems not to be a correct phenotyping scheme file!";
				message = String.format(message, tpcModel.getSourceFile());
				throw new Exception(message);
			}

		} catch (SAXException saxEx) {
			// the following step should ensure that an xml file
			// is closed even if an exception is thrown during reading
			new FileInputStream(xmlFile).close();
			throw saxEx;
		}
	}
}
