/*
Phenotyper - a tool for collecting phenotyping data using mobile terminals
Copyright (C) 2015,  jgremmels(at)bioinformatics.org

Phenotyper is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Contributors:
jgremmels(at)bioinformatics.org - initial API and implementation
*/

package mpimp.phenotypingschemecomposer.util;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import mpimp.phenotypingschemecomposer.db.EntityRecord;
import mpimp.phenotypingschemecomposer.db.TPCModel;
import mpimp.phenotypingschemecomposer.db.TestProgramEntityRecord;
import mpimp.phenotypingschemecomposer.db.ValueRecord;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class XMLWriter {

	// <?xml version="1.0" encoding="utf-16"?>
	// <TESTPROGRAM ID="" NAME="" VERSION="" OBJECT="" DESCRIPTION=""
	// OBJECT_TYPE="">
	// <PARAMETER ID="" NAME_E="" NAME_D="">
	// <ATTRIBUTE ID="" NAME_E="" NAME_D="" ORDER_NUMBER="">
	// <VALUE NAME_E="" NAME_D="" IMAGE_FILE="" />
	// </ATTRIBUTE>
	// <VALUE_GROUP ID="">
	// <ATTRIBUTE ID="" NAME_E="" NAME_D="" ORDER_NUMBER="">
	// <VALUE NAME_E="" NAME_D="" IMAGE_FILE="" />
	// </VALUE_GROUP>
	// </PARAMETER>
	// </TESTPROGRAM>

	private static final String tagNameTestProgram = "TESTPROGRAM";
	private static final String tagNameParameter = "PARAMETER";
	private static final String tagNameAttribute = "ATTRIBUTE";
	private static final String tagNameValue = "VALUE";
	private static final String tagNameValueGroup = "VALUE_GROUP";
	private static final String attributeNameID = "ID";
	private static final String attributeNameName = "NAME";
	private static final String attributeNameVersion = "VERSION";
	private static final String attributeNameObject = "OBJECT";
	private static final String attributeNameDescription = "DESCRIPTION";
	private static final String attributeNameObjectType = "OBJECT_TYPE";
	private static final String attributeNameName_E = "NAME_E";
	private static final String attributeNameName_D = "NAME_D";
	private static final String attributeNameOrderNumber = "ORDER_NUMBER";
	private static final String attributeNameImageFileName = "IMAGE_FILE";
	private static final String attributeNameNameSpace = "xmlns";
	private static final String attributeValueNameSpace = "http://tempuri.org/TestProgram.xsd";

	public static String writeXML(TPCModel tpcModel, String downloadTempDir)
			throws Exception {

		HashMap<EntityRecord, ArrayList<ValueRecord>> dataMap = arrangeData(tpcModel);

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document doc = builder.newDocument();

		Element testProgram = XMLUtilities.createTag(doc, tagNameTestProgram);

		testProgram.setAttribute(attributeNameNameSpace,
				attributeValueNameSpace);

		testProgram.setAttribute(attributeNameID, tpcModel
				.getCurrentTestProgramID().toString());
		testProgram.setAttribute(attributeNameName,
				tpcModel.getCurrentTestProgramName());
		testProgram.setAttribute(attributeNameVersion,
				tpcModel.getCurrentTestProgramVersion());
		testProgram.setAttribute(attributeNameObject,
				tpcModel.getCurrentTestProgramMessobjekt());
		testProgram.setAttribute(attributeNameDescription,
				tpcModel.getCurrentTestProgramDescription());
		testProgram.setAttribute(attributeNameObjectType, tpcModel
				.getCurrentTestObjectType().toString());

		doc.appendChild(testProgram);

		Set<EntityRecord> entityRecords = dataMap.keySet();
		for (EntityRecord currentEntityRecord : entityRecords) {
			Element parameterTag = XMLUtilities
					.createTag(doc, tagNameParameter);
			parameterTag.setAttribute(attributeNameID, currentEntityRecord
					.getOrganID().toString());
			parameterTag.setAttribute(attributeNameName_E,
					currentEntityRecord.getParameter());
			parameterTag.setAttribute(attributeNameName_D,
					currentEntityRecord.getParameterD());

			List<ValueRecord> valueRecords = dataMap.get(currentEntityRecord);
			HashMap<Integer, List<ValueRecord>> valueGroupMap = new HashMap<Integer, List<ValueRecord>>();
			for (ValueRecord currentValueRecord : valueRecords) {

				if (currentValueRecord.getValue_group() == null) {
					writeAttributeValueSection(doc, parameterTag,
							currentValueRecord);
				} else {
					// fill value group map first
					if (valueGroupMap.containsKey(currentValueRecord
							.getValue_group())) {
						valueGroupMap.get(currentValueRecord.getValue_group())
								.add(currentValueRecord);
					} else {
						List<ValueRecord> recordsForGroup = new ArrayList<ValueRecord>();
						recordsForGroup.add(currentValueRecord);
						valueGroupMap.put(currentValueRecord.getValue_group(),
								recordsForGroup);
					}
				}
			}
			// and write value groups - if there are any
			Set<Integer> valueGroupIds = valueGroupMap.keySet();
			for (Integer currentValueGroupId : valueGroupIds) {
				Element valueGroupTag = XMLUtilities.createTag(doc,
						tagNameValueGroup);
				valueGroupTag.setAttribute(attributeNameID,
						currentValueGroupId.toString());
				List<ValueRecord> recordsOfValueGroup = valueGroupMap
						.get(currentValueGroupId);
				for (ValueRecord currentRecordOfValueGroup : recordsOfValueGroup) {
					writeAttributeValueSection(doc, valueGroupTag,
							currentRecordOfValueGroup);
				}
				parameterTag.appendChild(valueGroupTag);
			}
			testProgram.appendChild(parameterTag);
		}
		String filePath = FileDownloadHelper.createXMLFilePath(tpcModel,
				downloadTempDir);
		writeDocumentToFile(doc, filePath);
		return filePath;
	}

	private static void writeAttributeValueSection(Document doc,
			Element parameterTag, ValueRecord currentValueRecord) {
		Element attributeTag = XMLUtilities.createTag(doc, tagNameAttribute);
		attributeTag.setAttribute(attributeNameID, currentValueRecord
				.getValueID().toString());
		attributeTag.setAttribute(attributeNameName_E,
				currentValueRecord.getAttribut_e());
		attributeTag.setAttribute(attributeNameName_D,
				currentValueRecord.getAttribut_d());
		Integer orderNumberInteger = currentValueRecord.getOrderNumber();
		if (orderNumberInteger != null) {
			attributeTag.setAttribute(attributeNameOrderNumber,
					orderNumberInteger.toString());
		} else {
			attributeTag.setAttribute(attributeNameOrderNumber, "0");
		}

		Element valueTag = XMLUtilities.createTag(doc, tagNameValue);
		valueTag.setAttribute(attributeNameName_E,
				currentValueRecord.getValue());
		valueTag.setAttribute(attributeNameName_D,
				currentValueRecord.getValue_d());
		valueTag.setAttribute(attributeNameImageFileName,
				currentValueRecord.getImageFileName());

		attributeTag.appendChild(valueTag);

		parameterTag.appendChild(attributeTag);
	}

	private static void writeDocumentToFile(Document doc, String path)
			throws Exception {
		FileOutputStream fos = new FileOutputStream(path);
		Transformer transformer = TransformerFactory.newInstance()
				.newTransformer();
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-16");
		transformer.transform(new DOMSource(doc), new StreamResult(fos));
		fos.close();
	}

	private static HashMap<EntityRecord, ArrayList<ValueRecord>> arrangeData(
			TPCModel tpcModel) {
		HashMap<EntityRecord, ArrayList<ValueRecord>> dataMap = new HashMap<EntityRecord, ArrayList<ValueRecord>>();
		ArrayList<TestProgramEntityRecord> tpeRecords = tpcModel
				.getTestProgramEntityRecords();
		for (TestProgramEntityRecord currentTPERecord : tpeRecords) {
			EntityRecord currentEntityRecord = tpcModel
					.getEntityRecordForId(currentTPERecord.getEntity());
			ValueRecord currentValueRecord = tpcModel
					.getValueRecordForId(currentTPERecord.getValue());
			currentValueRecord
					.setOrderNumber(currentTPERecord.getOrderNumber());
			if (dataMap.containsKey(currentEntityRecord)) {
				dataMap.get(currentEntityRecord).add(currentValueRecord);
			} else {
				ArrayList<ValueRecord> valueRecords = new ArrayList<ValueRecord>();
				valueRecords.add(currentValueRecord);
				dataMap.put(currentEntityRecord, valueRecords);
			}
		}
		return dataMap;
	}

}
